/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "feynman.h"

t_feynman_ecm *feynman_add_remote_ecm(t_feynman_remote_server *server)
{
    if(!server) return NULL;

    t_feynman_ecm *ecm = (t_feynman_ecm *)malloc(sizeof(t_feynman_ecm));

    ecm->caid = 0;
    ecm->provid = 0;
    ecm->sid = 0;
    ecm->ecm = NULL;
    ecm->ecm_len = 0;
    ecm->decrypt_status = FEYNMAN_ECMSTATUS_WAITING;
    ecm->dcw = NULL;
    ecm->dcw_len = 0;
    ecm->session = 0;
    ecm->client = NULL;
    ecm->card_id = 0;
    ecm->server_id = server->id;
    gettimeofday(&ecm->time_added, NULL);

    ecm->next = server->ecm_db;
    server->ecm_db = ecm;

    return ecm;
}

t_feynman_ecm *feynman_add_local_ecm(t_feynman_local_server *server)
{
    t_feynman_ecm *ecm = (t_feynman_ecm *)malloc(sizeof(t_feynman_ecm));

    ecm->caid = 0;
    ecm->provid = 0;
    ecm->sid = 0;
    ecm->ecm = NULL;
    ecm->ecm_len = 0;
    ecm->decrypt_status = FEYNMAN_ECMSTATUS_WAITING;
    ecm->dcw = NULL;
    ecm->dcw_len = 0;
    ecm->session = 0;
    ecm->server_id = server->id;
    ecm->client = NULL;

    //Allow ecms without a server, remember to free them
    if(server != NULL)
    {
        ecm->next = server->ecm_db;
        server->ecm_db = ecm;
    }

    return ecm;
}

 void feynman_set_ecm_data(t_feynman_ecm *ecm, unsigned char *ecm_data, int ecm_len)
{
    if(!ecm) return;
    if(!ecm_data) return;
    if(ecm_data == ecm->ecm) return;
    if(ecm->ecm != NULL) free(ecm->ecm);

    ecm->ecm = malloc(ecm_len);
    ecm->ecm_len = ecm_len;
    memcpy(ecm->ecm, ecm_data, ecm_len);
}

void feynman_set_ecm_dcw(t_feynman_ecm *ecm, unsigned char *dcw, int dcw_len)
{
    if(!ecm) return;
    if(!dcw) return;
    if(dcw == ecm->dcw) return;
    if(ecm->dcw != NULL) free(ecm->dcw);

    ecm->dcw = malloc(dcw_len);
    ecm->dcw_len = dcw_len;
    memcpy(ecm->dcw, dcw, dcw_len);

    int i;
    for(i=3; i<dcw_len; i+=4)
    {
        dcw[i] = dcw[i-1] + dcw[i-2] + dcw[i-3];
    }
}

void feynman_free_ecm(t_feynman_ecm *ecm)
{
    if(!ecm) return;

    if(ecm->dcw != NULL)
    {
        free(ecm->dcw);
    }
    if(ecm->ecm != NULL)
    {
        free(ecm->ecm);
    }
    free(ecm);
}

t_feynman_ecm *feynman_get_remote_ecm(t_feynman_remote_server *server, int session, int card_id)
{
    t_feynman_ecm *ecm;

    if(!server) return NULL;

    for(ecm=server->ecm_db; ecm!=NULL; ecm=ecm->next)
    {
        if(ecm->session == session && ecm->card_id == card_id)
        {
            return ecm;
        }
    }

    return NULL;
}

t_feynman_ecm *feynman_get_local_ecm(t_feynman_local_server *server, int session, int card_id)
{
    t_feynman_ecm *ecm;

    if(!server) return NULL;

    for(ecm=server->ecm_db; ecm!=NULL; ecm=ecm->next)
    {
        if(ecm->session == session && ecm->card_id == card_id)
        {
            return ecm;
        }
    }

    return NULL;
}

void feynman_del_remote_ecm(t_feynman_remote_server *server, t_feynman_ecm *ecm_to_del)
{
    if(!server) return;
    if(!ecm_to_del) return;

    t_feynman_ecm *ecm = server->ecm_db;
    t_feynman_ecm *prev = server->ecm_db;
    while(ecm != NULL)
    {
        t_feynman_ecm *tmp_ecm = ecm->next;
        if(ecm_to_del == ecm)
        {
            if(prev != NULL)
            {
                prev->next = ecm->next;
            }
            if(server->ecm_db == ecm)
            {
                server->ecm_db = ecm->next;
            }
            feynman_free_ecm(ecm);
            return;
        }
        prev = ecm;
        ecm = tmp_ecm;
    }
}

void feynman_del_remote_ecms(t_feynman_remote_server *server)
{
    if(!server) return;

    t_feynman_ecm *ecm = server->ecm_db;
    t_feynman_ecm *prev = server->ecm_db;
    while(ecm != NULL)
    {
        t_feynman_ecm *tmp_ecm = ecm->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = ecm->next;
            }
            if(server->ecm_db == ecm)
            {
                server->ecm_db = ecm->next;
            }
            feynman_free_ecm(ecm);
        }
        prev = ecm;
        ecm = tmp_ecm;
    }
}

void feynman_del_local_ecm(t_feynman_local_server *server, t_feynman_ecm *ecm_to_del)
{
    if(!server) return;
    if(!ecm_to_del) return;

    t_feynman_ecm *ecm = server->ecm_db;
    t_feynman_ecm *prev = server->ecm_db;
    while(ecm != NULL)
    {
        t_feynman_ecm *tmp_ecm = ecm->next;
        if(ecm_to_del == ecm)
        {
            if(prev != NULL)
            {
                prev->next = ecm->next;
            }
            if(server->ecm_db == ecm)
            {
                server->ecm_db = ecm->next;
            }
            feynman_free_ecm(ecm);
            return;
        }
        prev = ecm;
        ecm = tmp_ecm;
    }
}

void feynman_del_local_ecms(t_feynman_local_server *server)
{
    if(!server) return;

    t_feynman_ecm *ecm = server->ecm_db;
    t_feynman_ecm *prev = server->ecm_db;
    while(ecm != NULL)
    {
        t_feynman_ecm *tmp_ecm = ecm->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = ecm->next;
            }
            if(server->ecm_db == ecm)
            {
                server->ecm_db = ecm->next;
            }
            feynman_free_ecm(ecm);
        }
        prev = ecm;
        ecm = tmp_ecm;
    }
}
